﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Data;
using System.Windows;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Collections;

namespace Sample
{
    class CustomFilterDescriptor : CompositeFilterDescriptor
    {
        public CustomFilterDescriptor()
        {
            this.LogicalOperator = FilterCompositionLogicalOperator.Or;
        }

        private string member;
        public string Member
        {
            get
            {
                return this.member;
            }
            set
            {
                if (value != this.member)
                {
                    this.member = value;
                    this.OnPropertyChanged("Member");
                }
            }
        }

        private void ValuesCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (var item in e.NewItems)
                {
                    this.FilterDescriptors.Add(new FilterDescriptor(this.Member, FilterOperator.IsEqualTo, item));
                }
            }
            else if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                foreach (var item in e.OldItems)
                {
                    var descriptor = this.FilterDescriptors.OfType<FilterDescriptor>().Where((f) => f.Value == item).FirstOrDefault();
                    if (descriptor != null)
                    {
                        this.FilterDescriptors.Remove((IFilterDescriptor)descriptor);
                    }
                }
            }
            else if (e.Action == NotifyCollectionChangedAction.Reset)
            {
                this.FilterDescriptors.Clear();
            }
        }

        private FilterValues filter;
        public FilterValues Filter
        {
            get
            {
                return this.filter;
            }
            set
            {
                if (value != this.filter)
                {
                    if (this.filter != null)
                    {
                        this.filter.ValuesCollectionChanged -= this.ValuesCollectionChanged;
                    }

                    this.filter = value;
                    this.filter.ValuesCollectionChanged += this.ValuesCollectionChanged;
                }
            }
        }
    }

    public class FilterValues : DependencyObject
    {
        public ObservableCollection<string> Values
        {
            get { return (ObservableCollection<string>)GetValue(ValuesProperty); }
            set { SetValue(ValuesProperty, value); }
        }

        public static readonly DependencyProperty ValuesProperty =
            DependencyProperty.Register("Values", typeof(ObservableCollection<string>), typeof(FilterValues), new UIPropertyMetadata(new PropertyChangedCallback(ValuesChanged)));

        private static void ValuesChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (e.NewValue != null)
            {
                (e.NewValue as ObservableCollection<string>).CollectionChanged += (d as FilterValues).RaiseValuesCollectionChanged;
            }
            if (e.OldValue != null)
            {
                (e.OldValue as ObservableCollection<string>).CollectionChanged -= (d as FilterValues).RaiseValuesCollectionChanged;
            }
        }

        private void RaiseValuesCollectionChanged(object sender, NotifyCollectionChangedEventArgs args)
        {
            if (this.ValuesCollectionChanged != null)
            {
                this.ValuesCollectionChanged(sender, args);
            }
        }

        public event NotifyCollectionChangedEventHandler ValuesCollectionChanged;
    }
}
